@@ -1,8 +1,8 @@
-/*  $Id: BurgerSpaceClient.cpp,v 1.3 2010/09/16 03:22:22 sarrazip Exp $
+/*  $Id: BurgerSpaceClient.cpp,v 1.5 2013/02/25 05:04:43 sarrazip Exp $
     BurgerSpaceClient.cpp - Game client
 
     burgerspace - A hamburger-smashing video game.
-    Copyright (C) 2001-2010 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2001-2013 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -89,6 +89,7 @@ BurgerSpaceClient::BurgerSpaceClient(Role _role,
     levelNoAreaSize(),
     currentPauseMessage(),
     gameMode(IN_GAME),
+    slotDirCursorPos(1),
     theSoundMixer(NULL),
     useSound(_useSound),
     ingredientBouncesChunk(),
@@ -677,17 +678,25 @@ BurgerSpaceClient::getSlotNumber(SDLKey lastKeyPressed)
 */
 {
     if (lastKeyPressed >= SDLK_1 && lastKeyPressed <= SDLK_9)
-    {
-        int slotNum = int(lastKeyPressed - SDLK_1) + 1;
-        lastKeyPressed = SDLK_UNKNOWN;
-        return slotNum;
-    }
+        return int(lastKeyPressed - SDLK_1) + 1;
 
     if (lastKeyPressed == SDLK_ESCAPE)
-    {
-        lastKeyPressed = SDLK_UNKNOWN;
         return -2;
+
+    // Manage cursor: up and down arrows, and Enter.
+    //
+    if (lastKeyPressed == SDLK_UP)
+    {
+        slotDirCursorPos = (slotDirCursorPos + 7) % 9 + 1;
+        return -1;
     }
+    if (lastKeyPressed == SDLK_DOWN)
+    {
+        slotDirCursorPos = slotDirCursorPos % 9 + 1;
+        return -1;
+    }
+    if (lastKeyPressed == SDLK_RETURN)
+        return slotDirCursorPos;
 
     return -1;
 }
@@ -764,9 +773,9 @@ BurgerSpaceClient::showSlotDirectory()
 #ifndef _MSC_VER
     int fontHeight = getFontDimensions().y;
     int y = 11 * fontHeight;
-    writeStringXCentered("                           ", y += fontHeight);
-    writeStringXCentered("  No  Save Time            ", y += fontHeight);
-    writeStringXCentered("  --  -------------------  ", y += fontHeight);
+    writeStringXCentered("                               ", y += fontHeight);
+    writeStringXCentered("    No  Save Time              ", y += fontHeight);
+    writeStringXCentered("    --  -------------------    ", y += fontHeight);
 
     string dirname = getSavedGamesDir();
 
@@ -789,7 +798,10 @@ BurgerSpaceClient::showSlotDirectory()
                 strftime(fileTime, sizeof(fileTime), "%Y-%m-%d %H:%M:%S", &t);
             }
         }
-        snprintf(slotDesc, sizeof(slotDesc), "   %u  %-19s  ", slotNum, fileTime);
+        const char *leftCursor  = (slotNum == slotDirCursorPos ? "->" : "  ");
+        const char *rightCursor = (slotNum == slotDirCursorPos ? "<-" : "  ");
+        snprintf(slotDesc, sizeof(slotDesc), " %s  %u  %-19s %s ",
+                 leftCursor, slotNum, fileTime, rightCursor);
         writeStringXCentered(string(slotDesc), y += fontHeight);
     }
 
@@ -849,7 +861,7 @@ BurgerSpaceClient::doSaveDialog(SDLKey lastKeyPressed)
 void
 BurgerSpaceClient::drawLoadDialog()
 {
-    showDialogBox("     LOAD GAME: which slot? (1-9, Escape=cancel)     ");
+    showDialogBox("     LOAD GAME: which slot? (1-9, up/down/Enter, Escape=cancel)     ");
     showSlotDirectory();
 }
 
@@ -883,7 +895,7 @@ BurgerSpaceClient::doLoadDialog(SDLKey lastKeyPressed)
             {
                 int errorCode = serverInterface->loadGame(in);
                 if (errorCode != 0)
-                    msg << "Failed to load from slot" << slotNum;
+                    msg << "Failed to load from slot " << slotNum << ": error code #" << errorCode;
                 else
                 {
                     // Show the new game situation:
