@@ -1,4 +1,4 @@
-/*  $Id: server.cpp,v 1.1.2.65 2010/05/16 02:25:48 sarrazip Exp $
+/*  $Id: server.cpp,v 1.3 2010/09/16 03:22:22 sarrazip Exp $
     server.cpp - main() function for BurgerSpace server
 
     burgerspace - A hamburger-smashing video game.
@@ -25,6 +25,10 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <time.h>
+#include <errno.h>
+#include <limits.h>
+#include <sys/types.h>
+
 #include <string>
 #include <iostream>
 #include <iomanip>
@@ -32,15 +36,17 @@
 #include <vector>
 #include <map>
 
+#ifndef _MSC_VER
 #include <sys/time.h>
-#include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
 #include <unistd.h>
-#include <errno.h>
-#include <limits.h>
+#else
+#include <WinSock2.h>
+#include <ws2tcpip.h>
+#endif
 
 #ifdef HAVE_GETOPT_LONG
 #include <getopt.h>
@@ -448,12 +454,12 @@ public:
     {
     }
 
-    virtual bool saveGame(std::ostream &out)
+    virtual bool saveGame(std::ostream &/*out*/)
     {
         return false;  // save not supported in client-server mode
     }
 
-    virtual int loadGame(std::istream &in)
+    virtual int loadGame(std::istream &/*in*/)
     {
         return -1;  // load not supported in client-server mode
     }
@@ -649,7 +655,11 @@ BurgerSpaceCommandLineServer::processAvailableIncomingPackets()
     {
         memset(&from, '\0', sizeof(from));
         socklen_t length = sizeof(from);
-        ssize_t numBytesReceived = recvfrom(sock, buffer, sizeof(buffer), MSG_DONTWAIT, (sockaddr *) &from, &length);
+        int flags = 0;
+        #ifndef _MSC_VER
+        flags = MSG_DONTWAIT;
+        #endif
+        ssize_t numBytesReceived = recvfrom(sock, buffer, sizeof(buffer), flags, (sockaddr *) &from, &length);
         int e = errno;
         if (numBytesReceived < 0)  // if error or no packet available
         {
@@ -766,6 +776,10 @@ BurgerSpaceCommandLineServer::processAvailableIncomingPackets()
 }
 
 
+#ifdef _MSC_VER
+#undef main  // undo redefinition of "main" by SDL_main.h
+#endif
+
 int
 main(int argc, char *argv[])
 {
@@ -839,23 +853,38 @@ main(int argc, char *argv[])
     /*  Initialize the random number generator:
     */
     const char *s = getenv("SEED");
-    unsigned int seed = (s != NULL ? atol(s) : time(NULL));
+    unsigned int seed = (unsigned int) (s != NULL ? atol(s) : time(NULL));
     #ifndef NDEBUG
     cerr << "seed = " << seed << endl;
     cerr << "init-level-no = " << initLevelNo << endl;
     #endif
     srand(seed);
 
-
     try
     {
+        #ifdef _MSC_VER
+        WSADATA wsaData;
+        if (WSAStartup(MAKEWORD(2, 2), &wsaData) != 0)
+        {
+            cout << PROGRAM << ": WSAStartup failed: winsock error #" << WSAGetLastError() << endl;
+            return EXIT_FAILURE;
+        }
+
+        // Use destructor to ensure that winsock gets closed.
+        struct Cleanup { ~Cleanup() { WSACleanup(); } } cleanup;
+        #endif
+
         /*  Create a UDP server socket.
         */
         int sock = socket(AF_INET, SOCK_DGRAM, 0);
         if (sock < 0)
         {
+            #ifndef _MSC_VER
             int e = errno;
             cout << PROGRAM << ": failed to create server socket: " << strerror(e) << endl;
+            #else
+            cout << PROGRAM << ": failed to create server socket: winsock error #" << WSAGetLastError() << endl;
+            #endif
             return EXIT_FAILURE;
         }
         sockaddr_in server;
@@ -871,6 +900,11 @@ main(int argc, char *argv[])
         }
         cout << PROGRAM << ": server bound to port " << port << endl;
 
+        #ifdef _MSC_VER
+        u_long iMode = 1;
+        ioctlsocket(sock, FIONBIO, &iMode);  // non blocking mode
+        #endif
+
 
         /*  Create the "game logic" object.
         */
